# coding: utf-8

from django.contrib.auth.decorators import login_required
from django.db.models import get_model
from django.http import HttpResponseBadRequest, HttpResponseForbidden, HttpResponseRedirect
from django.views.decorators.http import require_POST
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from django.contrib import messages


@login_required
@require_POST
def save(request):
    """Save the field contents for the object"""

    def show_message(msg, level=messages.ERROR):
        messages.add_message(request, level, msg)
        return HttpResponseRedirect(request.POST.get('next', '/'))

    # TODO: check permissions better
    if not request.user.is_staff:
        return HttpResponseForbidden()
    try:
        app_label, model_name = request.POST['object_name'].split('.')
    except ValueError, e:
        return show_message(_('Cannot find the model'))

    model = get_model(app_label, model_name)
    if not model:
        return show_message(_('Cannot find the model'))

    field_name = request.POST.get('field_name')
    if (field_name and hasattr(model, field_name)):
        return show_message(_('No field_name parameter'))

    try:
        contents = request.POST['content']
    except KeyError:
        return show_message(_('No contents parameter provided'))

    try:
        obj = model.objects.get(pk=request.POST['object_pk'])
    except KeyError:
        return show_message(_('object_pk not provided'))
    except model.DoesNotExist:
        return show_message(_('Cannot find the object with this primary key'))

    setattr(obj, field_name, contents)
    obj.save()

    return show_message(_('Object saved'), messages.SUCCESS)



from django import template
from django.template.loader import render_to_string
from django.db import models

register = template.Library()

class AlohaNode(template.Node):

    def __init__(self, nodelist, obj=None, field_name=None, url=None):
        self.obj = template.Variable(obj) if obj else None
        self.field_name = template.Variable(field_name) if field_name else None
        self.nodelist = nodelist
        self.url = template.Variable(url) if url else None

    def render(self, context):
        inner_content = self.nodelist.render(context)
        if not context['request'].user.is_staff:
            return inner_content

        obj = self.obj.resolve(context) if self.obj else None
        if obj:
            assert isinstance(obj, models.Model), 'The object passed to aloha tag should be a django model'
        field_name = self.field_name.resolve(context) if self.field_name else None
        if field_name:
            assert isinstance(field_name, basestring), 'The field name passed to aloha tag should be a string'

        url = self.url.resolve(context) if self.url else None

        object_name = '%s.%s' % (obj._meta.app_label, obj._meta.object_name) if obj else None

        if obj:
            assert hasattr(obj, 'pk') and obj.pk, 'The model "%s" doesn\'t have a primary key, it cannot be used with aloha tag'
        context.update({
            'post_url': url,
            'object_name': object_name,
            'object_pk': obj.pk if obj else None,
            'field_name': field_name,
            'content': inner_content,
        })
        result = render_to_string('aloha/postform.html', context_instance=context)
        context.pop()
        return result
 

@register.tag('aloha')
def aloha(parser, token):
    """Django aloha main tag

    Use in your templates like this::

    {% load aloha_tags %}

    1. Option one - you want to edit a certain field on a model instance:

    {% aloha my_object_from_the_database 'fieldname_where_text_is' [url_to_save] %}
        {{ my_object_from_the_database.fieldname_where_text_is|safe }}
    {% endaloha %}

    If you want, you can add an optional "url" parameter to do custom
    POST processing. If not provided, aloha.views.save will be used.

    2. Option two - you want to edit arbitrary text, and provide and url to save:

    {% aloha [url_to_save_to] %}
       <p>Place here what you want to edit</p>
    {% endaloha %}
    """

    bits = token.split_contents()

    if len(bits) > 4:
        raise template.TemplateSyntaxError("Aloha tag takes maximum 4 arguments: %s given" % len(bits))

    elif len(bits) <= 2:
        url = bits[1] if len(bits) == 2 else None
        obj, field_name = None, None
    else:
        tag_name, obj, field_name = bits[:3]
        url = bits[3] if len(bits) == 4 else None
    nodelist = parser.parse(('endaloha',))
    parser.delete_first_token()
    return AlohaNode(nodelist, obj, field_name, url)



#!/usr/bin/env python

import os.path
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='django-aloha',
      version='0.0.1',
      description='django app to easily integrate editing with Aloha '\
          'WYSIWIG editor in django apps.',
      long_description=read('README.rst'),
      author='Igor Katson',
      author_email='igor.katson@gmail.com',
      url='https://bitbucket.org/descent/django-aloha',
      packages=['aloha', 'aloha.templatetags'],
      package_data={'aloha': ['templates/*.html',
                              'templates/*/*.html',
                              'templates/*/*/*.html',
                              ]},
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
          ],
     )
